#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.DecisionList {
  [Item("MDLEvaluator", "Description missing")]
  [StorableClass]
  public class MDLEvaluator : DecisionListEvaluator {

    public IValueLookupParameter<MDLCalculator> MDLCalculatorParameter {
      get { return (IValueLookupParameter<MDLCalculator>)Parameters["MDLCalculator"]; }
    }

    [StorableConstructor]
    protected MDLEvaluator(bool deserializing) : base(deserializing) { }
    protected MDLEvaluator(MDLEvaluator original, Cloner cloner)
      : base(original, cloner) {
    }
    public MDLEvaluator()
      : base() {
      Parameters.Add(new ValueLookupParameter<MDLCalculator>("MDLCalculator", ""));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MDLEvaluator(this, cloner);
    }

    public override IOperation Apply() {
      double penalty = 1;

      var dls = new DecisionListSolution(DecisionListParameter.ActualValue, ProblemDataParameter.ActualValue);

      if (dls.TrainingNumberOfAliveRules < SizePenaltyMinRulesParameter.ActualValue.Value) {
        penalty = (1 - 0.025 * (SizePenaltyMinRulesParameter.ActualValue.Value - dls.TrainingNumberOfAliveRules));
        if (penalty <= 0) penalty = 0.01;
        penalty *= penalty;
      }

      QualityParameter.ActualValue = new DoubleValue(MDLCalculatorParameter.ActualValue.CalculateFitness(dls) / penalty);
      return base.Apply();
    }
  }
}